﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace csharp_examples
{
    public partial class FormMain : Form
    {
        public FormMain()
        {
            InitializeComponent();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {

        }

        private void buttonType_Click(object sender, EventArgs e)
        {
            Object obj = new Button();
            String strObj = "obj = new Button()";
            Button button = null;
            if (checkBoxWrongType.Checked)
            {
                obj = new TextBox();
                strObj = "obj = new TextBox()";
            }
            OutLine = strObj;
            if (radioButtonAs.Checked)
            {
                OutLine = "obj as Button";
                button = obj as Button;
            }
            if (radioButtonCast.Checked)
            {
                try
                {
                    OutLine = "(Button)obj";
                    button = (Button)obj;
                } 
                catch (InvalidCastException ex) 
                {
                    OutLine = ex.Message;
                }
            }
            if (checkBoxNull.Checked)
            {
                try
                {
                    OutLine = "button.ToString()";
                    OutLine = button.ToString();
                }
                catch (NullReferenceException ex)
                {
                    OutLine = ex.Message;
                }
            }
            OutStars = "Test Type";
        }

        String OutLine
        {
            set { Out = value + "\r\n"; }
        }

        String OutStars
        {
            set { OutLine = "^^^^^^^^^^^^^^^^^^^^^^^^" + value + "^^^^^^^^^^^^^^^^^^^^^^^^\r\n"; }
        }

        String Out
        {
            set { 
                textBoxOut.Text += value;
                textBoxOut.SelectionStart = textBoxOut.TextLength;
                textBoxOut.ScrollToCaret();
            }
        }

        private void buttonTest_Click(object sender, EventArgs e)
        {
            Object obj = new Form();
            String strObj = "obj = new Form()";
            if (radioButtonContainer.Checked)
            {
                obj = new ContainerControl();
                strObj = "new ContainerControl()";
            }
            OutLine = strObj;
            if (radioButtonIs.Checked)
            {
                OutLine = "obj is ContainerControl";
                IsOrIsNot(obj is ContainerControl, "Container Control");
            }
            else if (radioButtonEquals.Checked)
            {
                OutLine = "obj.GetType().Name.Equals(\"ContainerControl\")";          
                IsOrIsNot(obj.GetType().Name.Equals("ContainerControl"), "Container Control");
            }
            else if (radioButtonSubclass.Checked)
            {
                OutLine = "ContainerControl contain = new ContainerControl();";
                OutLine = "obj.GetType().IsSubclassOf(contain.GetType())";
                ContainerControl contain = new ContainerControl();
                IsOrIsNot(obj.GetType().IsSubclassOf(contain.GetType()), "Container Control");
            }
            else if (radioButtonSubEqual.Checked)
            {
                OutLine = "ContainerControl contain = new ContainerControl();";
                OutLine = "obj.GetType().Name.Equals(\"ContainerControl\") || obj.GetType().IsSubclassOf(contain.GetType())";
                ContainerControl contain = new ContainerControl();
                IsOrIsNot(obj.GetType().Name.Equals("ContainerControl") || 
                    obj.GetType().IsSubclassOf(contain.GetType()), "Container Control");
            }
            OutStars = "Test Subtype";
        }

        void IsOrIsNot(bool state, String comment)
        {
            String not = "not ";
            if (state) not = "";
            OutLine = "Is " + not + comment;
        }

        private void buttonVirtual_Click(object sender, EventArgs e)
        {
            if (radioButtonBase.Checked)
            {
                BaseClass baseClass = new BaseClass();
                OutLine = "BaseClass baseClass = new BaseClass();";
                OutLine = baseClass.NonVirtual();
                OutLine = baseClass.Virtual();
                OutStars = "BaseClass Method Calls";
            }
            else if (radioButtonExtend.Checked)
            {
                ExtendedClass extenedClass = new ExtendedClass();
                OutLine = "ExtendedClass extenedClass = new BaseClass();";
                OutLine = extenedClass.NonVirtual();
                OutLine = extenedClass.Virtual();
                OutStars = "ExtendedClass Method Calls";
            }
            else if (radioButtonBaseExtend.Checked)
            {
                BaseClass extenedClass = new ExtendedClass();
                OutLine = "ExtendedClass extenedClass = new BaseClass();";
                OutLine = extenedClass.NonVirtual();
                OutLine = extenedClass.Virtual();
                OutStars = "ExtendedClass as BaseClass Method Calls";
            }
        }

        private void buttonViewBase_Click(object sender, EventArgs e)
        {
            TextReader reader = new StreamReader(@"..\..\BaseClass.cs");
            Out = reader.ReadToEnd();
            OutStars = "Base Class";
        }

        private void buttonViewExtended_Click(object sender, EventArgs e)
        {
            TextReader reader = new StreamReader(@"..\..\ExtendedClass.cs");
            Out = reader.ReadToEnd();
            OutStars = "Extended Class";
        }

        private void DoDefault(Form form, String str, Point pt)
        {
            OutLine = "Default: form by ref, str by ref, pt by value";
            form.Text = "New and improved text";
            OutLine = "\tform.Text = \"New and improved text!\"";
            str = "I've changed!";
            OutLine = "\tstr = \"I've changed!\"";
            pt.X = pt.X + 10;
            OutLine = "\tpt.X = pt.X + 10";
        }

        private void DoReference(ref Form form, ref String str, ref Point pt)
        {
            OutLine = "Default: form by ref to ref, str by ref to ref, pt by ref";
            form = new Form();
            form.Text = "I am a new form with a new title";
            OutLine = "\tform = new Form()";
            OutLine = "\tform.Text = \"I am a new form with a new title\"";
            str = "I've changed!";
            OutLine = "\tstr = \"I've changed!\"";
            pt.X = pt.X + 10;
            OutLine = "\tpt.X = pt.X + 10";
        }

        private void buttonDefault_Click(object sender, EventArgs e)
        {
            Form form = new Form();
            form.Text = "I am the original text";
            OutLine = "form.Text = \"I am the original text\"";
            String str = "I will never change";
            OutLine = "String str = \"I will never change\"";
            Point pt = new Point(0, 0);
            OutLine = "Point pt = new Point(0, 0)";
            DoDefault(form, str, pt);
            OutLine = String.Format("After call: form.Text='{0}', str='{1}', pt={2}", form.Text, str, pt);
            OutStars = "Default Parameter Passing";
        }

        private void buttonReference_Click(object sender, EventArgs e)
        {
            Form form = new Form();
            form.Text = "I am the original text";
            OutLine = "form.Text = \"I am the original text\"";
            String str = "I will never change";
            OutLine = "String str = \"I will never change\"";
            Point pt = new Point(0, 0);
            OutLine = "Point pt = new Point(0, 0)";
            DoReference(ref form, ref str, ref pt);
            OutLine = String.Format("After call: form.Text='{0}', str='{1}', pt={2}", form.Text, str, pt);
            OutStars = "Reference Parameter Passing";
        }
    }
}
